@@ -4,6 +4,7 @@ import argparse
 import os
 import subprocess
 import sys
+import time
 
 # helper function to print log messages depending on verbosity
 def log (level, *msg):
@@ -13,13 +14,14 @@ def log (level, *msg):
 sys.dont_write_bytecode = True
 
 # helper function to run and log processes
-def runprog (name, progargs, exit_on_failure=True):
+def runprog (name, progargs, env=None, exit_on_failure=True):
     log(1, 'Running', name)
     proc = subprocess.run(
         progargs,
         timeout = 300,
         capture_output = True,
         text = True,
+        env = env,
     )
     if proc.returncode != 0:
         log(0,"#"*72)
@@ -35,7 +37,7 @@ def runprog (name, progargs, exit_on_failure=True):
     return proc
 
 def locale_debug(name):
-    if args.verbose >= 3:
+    if args.verbose >= 2:
         log(2, name)
         log(2, 'output of /usr/bin/locale:')
         subprocess.run(['/usr/bin/locale'])
@@ -58,8 +60,8 @@ parser = argparse.ArgumentParser(
 parser.add_argument(
     '--default-tests',
     action=argparse.BooleanOptionalAction,
-    default=True,
-    help='Run the default tests not listed anywhere else. (default: yes)',
+    default=False,
+    help='Run the default tests not listed anywhere else. (default: no)',
 )
 
 parser.add_argument(
@@ -79,8 +81,8 @@ parser.add_argument(
 parser.add_argument(
     '--requires-apt-mark-manual',
     action=argparse.BooleanOptionalAction,
-    default=True,
-    help='Run tests that do "apt-mark manual" on certain packages. (default: yes)',
+    default=False,
+    help='Run tests that do "apt-mark manual" on certain packages. (default: no)',
 )
 
 parser.add_argument(
@@ -111,6 +113,15 @@ parser.add_argument(
     help='Print the list of targets without actually running them',
 )
 
+parser.add_argument(
+    '--check-test-lists',
+    action=argparse.BooleanOptionalAction,
+    default=False,
+    help='Check if all the hardcoded integration tests from the other \
+          flags (e.g. --requires-root, --failing, ,etc.) still exist \
+          in the ansible-core test suite. Defaults to no. When set, \
+          skips everything else.',
+)
 parser.add_argument(
     '--verbose', '-v',
     action='count',
@@ -121,35 +132,22 @@ parser.add_argument(
 
 args = parser.parse_args()
 
+# Don't set up test env when checking test lists
+if args.check_test_lists:
+    args.setup = False
+
 locale_debug('locale before setting os.environ:')
 os.environ['LANG'] = 'en_US.UTF-8'
 locale_debug('locale after setting os.environ:')
 
-if args.setup is True:
+if args.setup is True and args.dry_run == False:
     proc = runprog('testbed-setup.sh', ['sudo', './debian/tests/testbed-setup.sh'])
-    log(2,"#### STDOUT ####")
-    log(2, proc.stdout)
-    log(2, "#### STDERR ####")
-    log(2, proc.stderr)
+    log(0,"#### STDOUT ####")
+    log(0, proc.stdout)
+    log(0, "#### STDERR ####")
+    log(0, proc.stderr)
     locale_debug('locale after running testbed-setup.sh:')
 
-# integration tests requiring root in some form
-integration_requires_root = {
-    'ansible-vault',    # ignores --local and tries to use venv
-    'apt_key',          # add/removes apt keys
-    'blockinfile',      # setup_remote_tmp_dir handler fails (hidden output)
-    'callback_default', # checks for an error that has root's homedir
-    'debconf',          # Writes to debconf database
-    'gathering',        # writes to /etc/ansible/facts.d/
-    'group',            # wants to add/remove systemd groups
-    'keyword_inheritance', # requires sudo
-    #'module_defaults',  # requires sudo
-    'noexec',           # calls mount
-    #'omit',             # requires sudo
-    'raw',              # requires su
-    'systemd',          # disables/enables services
-}
-
 # integration tests requiring a running ssh server
 integration_requires_ssh = {
     'become_unprivileged',
@@ -159,6 +157,7 @@ integration_requires_ssh = {
     'delegate_to',
     'fetch',
     'module_tracebacks',
+    'ssh_agent',
 }
 
 # integration tests requiring root because the apt module is used to
@@ -180,27 +179,29 @@ integration_fails_on_pip = {
 }
 
 integration_failing = {
-    'ansible-galaxy-collection-scm':  "ansible-core on PyPI depends on resolvelib<1.1.0",
     'ansible-galaxy-role':            'dict object has no attribute lnk_source',  ## needs upstream fix?
     'ansible-test-docker':            "pwsh doesn't exist in Debian yet",
     'ansible-test':                   'installs and runs python libs from remote',
+    'ansible-test-debugging':         'requires running built binary instead of package binary',
+    'ansible-test-installed':         'checks only valid if source tree has bin/ directory',
     'ansible-test-sanity':            'checks are only valid for the source tree',
     'ansible-test-units-forked':      '?????',
+    'ansible-test-vendoring':         'Should test with ./bin/ansible-test which was removed in v2.18.0', ## needs upstream fix
+    'deb822_repository':              'setup_deb_repo seems to generate a repo with hashsum mismatch',
     'facts_d':                        'seems to read an unreadable problem without error', ## needs upstream fix
     'infra':                          'requires hacking/test-module.py not present',
-    'interpreter_discovery_python':   'detects /usr/bin/python3.11, expect python3, detects os_version 12.6, expects it to compare > 10',
-#    'preflight_encoding':             'fails due to missing en_US.UTF-8', # workaround in testbed-setup.sh
-    'remote_tmp':                     'Will often show false positive on: "Test tempdir is removed", needs upstream fixing',
-    'service_facts':                  "Version comparison failed: '<' not supported between instances of 'str' and 'int'",    # writes to /usr/sbin/
-#    'tags':                           'fails due to missing en_US.UTF-8', # workaround in testbed-setup.sh
-    'template_jinja2_non_native':     'no need to test against latest jinja2',
+    'mount_facts':                    'makes assumptions about static/dynamic mounts without assuring they exist',
+    'packaging_cli-doc':              'checks only valid if source tree has bin/ directory',
+    'unarchive':                      'two unarchive tasks in a row, second one reports changed',
 }
 
 if subprocess.check_output(['dpkg', '--print-architecture'], text=True).rstrip('\n') not in {'amd64', 'ppc64el'}:
     integration_failing['binary_modules_posix'] = 'needs statically-built helloworld_linux_* from ci-files.testing.ansible.com'
 
 # work around autopkgtest providing the source tree owned by a different user
-runprog('git config hack', ['git', 'config', '--global', '--add', 'safe.directory', '*'])
+if args.setup:
+    runprog('git config hack for normal user', ['git', 'config', '--global', '--add', 'safe.directory', '*'])
+    runprog('git config hack for root', ['sudo', 'git', 'config', '--global', '--add', 'safe.directory', '*'])
 
 pyver = str(sys.version_info.major) + '.' + str(sys.version_info.minor)
 
@@ -211,7 +212,8 @@ succeeded_tests = []
 # retrieve a list of all integration tests
 all_targets_cmd = runprog(
     'ansible-test to retrieve list of targets',
-    ['./bin/ansible-test', 'integration', '--list-targets'],
+    ['ansible-test', 'integration', '--list-targets', '--allow-root', '--allow-destructive'],
+    env={**os.environ, 'PYTHONPATH': os.path.join(os.getcwd(), 'test', 'lib')},
 )
 
 # Compile list of all targets
@@ -219,13 +221,31 @@ all_targets = set(all_targets_cmd.stdout.splitlines())
 
 default_targets = list(
     all_targets
-    - integration_requires_root
     - integration_requires_ssh
     - integration_requires_apt_mark_manual
     - integration_fails_on_pip
     - set(integration_failing.keys())
 )
 
+def check_if_list_in_all_targets( l1, name):
+    l = l1 - all_targets
+    global overall_test_rc # hack to change it out of func
+    if len(l) !=0:
+        log(0, name, 'has elements not in all_targets list!')
+        for i in l:
+            log(0, ' ', i)
+        overall_test_rc = 1
+        log(0, '  ^^^^ consider removing those.')
+    else:
+        log(0, name, 'looks good.')
+
+if args.check_test_lists:
+    check_if_list_in_all_targets(integration_requires_ssh, 'integration_requires_ssh')
+    check_if_list_in_all_targets(integration_requires_apt_mark_manual, 'integration_requires_apt_mark_manual')
+    check_if_list_in_all_targets(integration_fails_on_pip, 'integration_fails_on_pip')
+    check_if_list_in_all_targets(integration_failing.keys(), 'integration_failing')
+    sys.exit(overall_test_rc)
+
 # compile a list of targets to run, depending on CLI parameters
 targets = []
 skipped = []
@@ -235,11 +255,6 @@ if args.default_tests is True:
 else:
     skipped.extend(default_targets)
 
-if args.requires_root is True:
-    targets.extend(integration_requires_root)
-else:
-    skipped.extend(integration_requires_root)
-
 if args.requires_ssh is True:
     targets.extend(integration_requires_ssh)
 else:
@@ -260,6 +275,9 @@ if args.failing is True:
 else:
     skipped.extend(integration_failing)
 
+# TODO: remove any tests that we're skipping from the list
+# e.g. "cli" is in requires-root and also requires-ssh
+
 targets.sort()
 skipped.sort()
 
@@ -267,49 +285,101 @@ for i in targets:
 
     if args.dry_run is True:
         log(1, 'Would run ansible-test in', i)
-        skipped.append(i)
+        succeeded_tests.append({
+          'name': i,
+          'time': 0
+        })
         continue
 
-    print ("\n" + "#"*72, flush=True)
-    print ("#### Running integration tests in", i, flush=True)
-    print ("#"*72, flush=True)
+    log(0, "\n" + "#"*72)
+    log(0, "#### Running integration tests in", i)
+    log(0, "#"*72)
 
-    proc = subprocess.run([
-        './bin/ansible-test',
+    cmdlist = [
+        'ansible-test',
         'integration',
+        '--allow-root',
+        '--allow-destructive',
+        '--venv-system-site-packages',
         '--python-interpreter',
         '/usr/bin/python3',
         '--python',
         pyver,
         '--local',
         '--color', 'yes',
-        i
-    ])
+        i,
+        # '-vvvv', # uncomment for more test output
+    ]
+
+    env={
+        **os.environ,
+        'PYTHONPATH': os.path.join(os.getcwd(), 'test', 'lib'),
+    }
+
+    # Check if test requires root privileges via aliases file
+    needs_root = False
+    with open(os.path.join('./test/integration/targets', i, 'aliases')) as f:
+        for line in f.read().splitlines():
+            if line == 'needs/root':
+                needs_root = True
+
+    if needs_root is True:
+        cmdlist = ['sudo', '--preserve-env=PYTHONPATH'] + cmdlist
+
+    # For debugging permission errors on writing results output
+    if args.verbose >= 3:
+        subprocess.run(['ls', '-ld', './test/'])
+        subprocess.run(['ls', '-ld', './test/results/'])
+        subprocess.run(['ls', '-ld', './test/results/.tmp/'])
+        subprocess.run(['ls', '-ld', './test/results/.tmp/output_dir/'])
+
+    # fix any permission errors due to some root tests not cleaning up correctly
+    subprocess.run(['sudo', 'chown', '-R', str(os.getuid()) + ':' + str(os.getgid()), './test'])
+
+    start = time.time()
+    proc = subprocess.run(cmdlist, env=env)
+    end = time.time()
 
     if proc.returncode != 0:
-        failed_tests.append(i)
+        failed_tests.append({
+          'name': i,
+          'time': int( end - start)
+        })
         overall_test_rc = proc.returncode
     else:
-        succeeded_tests.append(i)
+        succeeded_tests.append({
+          'name': i,
+          'time': int( end - start)
+        })
 
 if overall_test_rc != 0:
-    print ("#"*72, flush=True)
-    print ("#### failed tests are:", flush=True)
+    log(0, "#"*72)
+    log(0, "#### failed tests are:")
     for i in failed_tests:
-        print ("####", i, flush=True)
-    print ("#"*72, flush=True)
+        log(0, "####", i['name'])
+    log(0, "#"*72)
+
+if (args.default_tests is False
+  and (args.failing or args.fails_on_pip or args.requires_ssh)
+  and len(succeeded_tests) > 0):
+    log(0, "#"*72)
+    log(0, "#### succeeded tests are:")
+    for i in succeeded_tests:
+        log(0, "####", i['name'])
+    log(0, "Consider removing these ^^^ tests from the failing list!")
+    log(0, "#"*72)
 
 log(1, '### TEST SUMMARY ###')
 log(1, 'succeeded tests:', len(succeeded_tests))
 log(1, 'failed tests:', len(failed_tests))
 log(1, 'skipped tests:', len(skipped))
 
-log(2, '### succeed test list:')
+log(2, '### succeeded test list:')
 for i in succeeded_tests:
-    log(2, i)
+    log(2, '{:<30} (elapsed time: {:>3}s)'.format( i['name'], i['time'] ) )
 log(2, '### failed test list:')
 for i in failed_tests:
-    log(2, i)
+    log(2, '{:<30} (elapsed time: {:>3}s)'.format( i['name'], i['time'] ) )
 log(2, '### skipped test list:')
 for i in skipped:
     log(2, i)
